@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2012 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,19 +29,16 @@
 #include "weather.h"
 
 #include "weather-search.h"
-#include "weather-http.h"
 #include "weather-debug.h"
 
 #define BORDER 8
 
 
 typedef struct {
-    const gchar *proxy_host;
-    gint proxy_port;
     void (*cb) (const gchar *loc_name,
                 const gchar *lat,
                 const gchar *lon,
-                const unit_systems unit_system,
+                const units_config *units,
                 gpointer user_data);
     gpointer user_data;
 } geolocation_data;
@@ -79,9 +76,8 @@ sanitize_str(const gchar *str)
 
 
 static void
-cb_searchdone(const gboolean succeed,
-              gchar *received,
-              const size_t len,
+cb_searchdone(SoupSession *session,
+              SoupMessage *msg,
               gpointer user_data)
 {
     search_dialog *dialog = (search_dialog *) user_data;
@@ -94,22 +90,11 @@ cb_searchdone(const gboolean succeed,
 
     gtk_widget_set_sensitive(dialog->find_button, TRUE);
 
-    if (!succeed || received == NULL)
-        return;
-
-    if (g_utf8_validate(received, -1, NULL)) {
-        /* force parsing as UTF-8, the XML encoding header may lie */
-        doc = xmlReadMemory(received, strlen(received), NULL, "UTF-8", 0);
-    } else {
-        doc = xmlParseMemory(received, strlen(received));
-    }
-    g_free(received);
-
+    doc = get_xml_document(msg);
     if (!doc)
         return;
 
     cur_node = xmlDocGetRootElement(doc);
-
     if (cur_node) {
         for (cur_node = cur_node->children; cur_node;
              cur_node = cur_node->next) {
@@ -126,10 +111,8 @@ cb_searchdone(const gboolean succeed,
                 xml_place_free(place);
                 place = NULL;
             }
-
         }
     }
-
     xmlFreeDoc(doc);
 
     if (found > 0)
@@ -178,14 +161,13 @@ search_cb(GtkWidget *widget,
     gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog->dialog),
                                       GTK_RESPONSE_ACCEPT, FALSE);
 
-    url = g_strdup_printf("/search?q=%s&format=xml", sane_str);
+    url = g_strdup_printf("http://nominatim.openstreetmap.org/"
+                          "search?q=%s&format=xml", sane_str);
     g_free(sane_str);
 
     gtk_tree_view_column_set_title(dialog->column, _("Searching..."));
-    g_message("getting http://nominatim.openstreetmap.org%s", url);
-    weather_http_receive_data("nominatim.openstreetmap.org", url,
-                              dialog->proxy_host, dialog->proxy_port,
-                              cb_searchdone, dialog);
+    g_message(_("getting %s"), url);
+    weather_http_queue_request(dialog->session, url, cb_searchdone, dialog);
     g_free(url);
 }
 
@@ -202,8 +184,7 @@ pass_search_results(GtkTreeView *tree_view,
 
 search_dialog *
 create_search_dialog(GtkWindow *parent,
-                     gchar *proxy_host,
-                     gint proxy_port)
+                     SoupSession *session)
 {
     search_dialog *dialog;
     GtkWidget *dialog_vbox, *vbox, *hbox, *scroll, *frame;
@@ -214,8 +195,7 @@ create_search_dialog(GtkWindow *parent,
     if (!dialog)
         return NULL;
 
-    dialog->proxy_host = proxy_host;
-    dialog->proxy_port = proxy_port;
+    dialog->session = session;
 
     dialog->dialog =
         xfce_titled_dialog_new_with_buttons(_("Search location"),
@@ -334,80 +314,85 @@ free_search_dialog(search_dialog *dialog)
 }
 
 
-static unit_systems
-get_preferred_unit_system(const gchar *country_code)
+static units_config *
+get_preferred_units(const gchar *country_code)
 {
+    units_config *units;
+
+    if (G_UNLIKELY(country_code == NULL))
+        return NULL;
+
+    units = g_slice_new0(units_config);
+    if (G_UNLIKELY(units == NULL))
+        return NULL;
+
     /* List gathered from http://www.maxmind.com/app/iso3166 */
-    if (country_code == NULL)
-        return METRIC;
-    else if (!strcmp(country_code, "US"))  /* United States */
-        return IMPERIAL;
-    else if (!strcmp(country_code, "GB"))  /* United Kingdom */
-        return IMPERIAL;
-    else if (!strcmp(country_code, "LR"))  /* Liberia */
-        return IMPERIAL;
-    else if (!strcmp(country_code, "MM"))  /* Myanmar(Burma) */
-        return IMPERIAL;
-    return METRIC;
+    if (!strcmp(country_code, "US") ||        /* United States */
+        !strcmp(country_code, "GB") ||        /* United Kingdom */
+        !strcmp(country_code, "JM") ||        /* Jamaica */
+        !strcmp(country_code, "LR") ||        /* Liberia */
+        !strcmp(country_code, "MM")) {        /* Myanmar(Burma) */
+        units->pressure = PSI;
+        units->windspeed = MPH;
+        units->precipitation = INCHES;
+        units->altitude = FEET;
+    } else {
+        units->pressure = HECTOPASCAL;
+        units->windspeed = KMH;
+        units->precipitation = MILLIMETERS;
+        units->altitude = METERS;
+    }
+
+    if (!strcmp(country_code, "US") ||        /* United States */
+        !strcmp(country_code, "JM")) {        /* Jamaica */
+        units->temperature = FAHRENHEIT;
+    } else {
+        units->temperature = CELSIUS;
+    }
+
+    if (!strcmp(country_code, "RU"))          /* Russian Federation */
+        units->pressure = TORR;
+
+    if (!strcmp(country_code, "US"))          /* United States */
+        units->apparent_temperature = WINDCHILL_HEATINDEX;
+    else if (!strcmp(country_code, "CA"))     /* Canada */
+        units->apparent_temperature = WINDCHILL_HUMIDEX;
+    else if (!strcmp(country_code, "AU"))     /* Australia */
+        units->apparent_temperature = STEADMAN;
+
+    return units;
 }
 
 
 static void
-cb_geolocation(const gboolean succeed,
-               gchar *received,
-               const size_t len,
+cb_geolocation(SoupSession *session,
+               SoupMessage *msg,
                gpointer user_data)
 {
     geolocation_data *data = (geolocation_data *) user_data;
-    xmlDoc *doc;
     xml_geolocation *geo;
-    gchar *full_loc, *p;
-    unit_systems unit_system;
-    gsize length;
+    gchar *full_loc;
+    units_config *units;
 
-    if (!succeed || received == NULL) {
-        data->cb(NULL, NULL, NULL, METRIC, data->user_data);
-        g_free(data);
-        return;
-    }
-
-    /* hack for geoip.xfce.org, which return no content-length */
-    p = strstr(received, "</Response>");
-    if (p != NULL)
-        length = p - received + strlen("</Response>");
-    else
-        length = strlen(received);
-
-    if (g_utf8_validate(received, -1, NULL)) {
-        /* force parsing as UTF-8, the XML encoding header may lie */
-        doc = xmlReadMemory(received, length, NULL, "UTF-8", 0);
-    } else
-        doc = xmlParseMemory(received, length);
-    g_free(received);
-
-    if (!doc) {
-        data->cb(NULL, NULL, NULL, METRIC, data->user_data);
-        g_free(data);
-        return;
-    }
-
-    geo = parse_geolocation(xmlDocGetRootElement(doc));
-    xmlFreeDoc(doc);
+    geo = (xml_geolocation *)
+        parse_xml_document(msg, (XmlParseFunc) parse_geolocation);
     weather_dump(weather_dump_geolocation, geo);
 
-    if (geo == NULL) {
-        data->cb(NULL, NULL, NULL, METRIC, data->user_data);
+    if (!geo) {
+        data->cb(NULL, NULL, NULL, NULL, data->user_data);
         g_free(data);
         return;
     }
 
-    if (geo->country_name && geo->city)
+    if (geo->country_name && geo->city && strcmp(geo->city, "(none)"))
         if (geo->country_code && !strcmp(geo->country_code, "US") &&
             geo->region_name)
             full_loc = g_strdup_printf("%s, %s", geo->city, geo->region_name);
         else
             full_loc = g_strdup_printf("%s, %s", geo->city, geo->country_name);
-    else if (geo->country_name)
+    else if (geo->region_name && strcmp(geo->region_name, "(none)"))
+        full_loc = g_strdup(geo->region_name);
+    else if (geo->country_name && strcmp(geo->country_name, "(none)"))
         full_loc = g_strdup(geo->country_name);
     else if (geo->latitude && geo->longitude) {
         /*
@@ -419,25 +404,26 @@ cb_geolocation(const gboolean succeed,
     } else
         full_loc = NULL;
 
-    unit_system = get_preferred_unit_system(geo->country_code);
-
-    data->cb(full_loc, geo->latitude, geo->longitude, unit_system, data->user_data);
+    units = get_preferred_units(geo->country_code);
+    weather_dump(weather_dump_units_config, units);
+    data->cb(full_loc, geo->latitude, geo->longitude, units, data->user_data);
+    g_slice_free(units_config, units);
     xml_geolocation_free(geo);
     g_free(full_loc);
     g_free(data);
 }
 
 
-void weather_search_by_ip(const gchar *proxy_host,
-                          gint proxy_port,
+void weather_search_by_ip(SoupSession *session,
                           void (*gui_cb) (const gchar *loc_name,
                                           const gchar *lat,
                                           const gchar *lon,
-                                          const unit_systems unit_system,
+                                          const units_config *units,
                                           gpointer user_data),
                           gpointer user_data)
 {
     geolocation_data *data;
+    const gchar *url = "http://geoip.xfce.org/";
 
     if (!gui_cb)
         return;
@@ -445,10 +431,7 @@ void weather_search_by_ip(const gchar *proxy_host,
     data = g_new0(geolocation_data, 1);
     data->cb = gui_cb;
     data->user_data = user_data;
-    data->proxy_host = proxy_host;
-    data->proxy_port = proxy_port;
 
-    g_message("getting http://geoip.xfce.org/");
-    weather_http_receive_data("geoip.xfce.org", "/", proxy_host, proxy_port,
-                              cb_geolocation, data);
+    g_message(_("getting %s"), url);
+    weather_http_queue_request(session, url, cb_geolocation, data);
 }
